package net.sf.itcb.addons.monitoring.track.web.vaadin.interceptor;

import javax.portlet.PortletRequest;
import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Required;

import net.sf.itcb.addons.monitoring.track.MonitoringTransactionManager;
import net.sf.itcb.addons.monitoring.track.bean.ItcbTrackBean;
import net.sf.itcb.addons.monitoring.track.bean.LayerType;
import net.sf.itcb.common.web.vaadin.ItcbVaadinApplication;
import net.sf.itcb.common.web.vaadin.interceptor.request.ItcbRequestInterceptor;
import net.sf.itcb.common.web.vaadin.interceptor.request.ItcbRequestInterceptor.ItcbRequestInterceptorExceptionListener;

/**
 * This interceptor's aim is to track each request received by the portlet or the servlet.<br/>
 * It can be usefull in development mode but should be used in Production mode.<br/>
 * In production mode, {@link TrackerNoTrackRequestInterceptor} should be used instead in order to only clean transactionId before leaving the request.
 * @author Pierre Le Roux
 */
public class TrackerRequestInterceptor implements ItcbRequestInterceptor, ItcbRequestInterceptorExceptionListener {

	private static final String HANDLE_REQUEST = "handleRequest";
	private static final String NET_SF_ITCB_ADDONS_MONITORING_TRACK_PORTLET_INTERCEPTOR_TRACKER_REQUEST_INTERCEPTOR = "net.sf.itcb.addons.monitoring.track.web.vaadin.interceptor.TrackerRequestInterceptor";

	MonitoringTransactionManager monitoringTransactionManager;
	
	@Required
	public void setMonitoringTransactionManager(MonitoringTransactionManager monitoringTransactionManager) {
		this.monitoringTransactionManager=monitoringTransactionManager;
	}
		
	@Override
	public void handleRequestStart(ItcbVaadinApplication application,
			HttpServletRequest servletRequest) {
		  
		ItcbTrackBean itcbTrackBean = new ItcbTrackBean();
		itcbTrackBean.setClassName(NET_SF_ITCB_ADDONS_MONITORING_TRACK_PORTLET_INTERCEPTOR_TRACKER_REQUEST_INTERCEPTOR);
	    itcbTrackBean.setMethod(HANDLE_REQUEST);
	    itcbTrackBean.setLayerType(LayerType.VAADIN_REQUEST);
		itcbTrackBean.setTrackIfNoChild(false);
		monitoringTransactionManager.setCurrentTrackBean(itcbTrackBean);

	}

	@Override
	public void handleRequestStart(ItcbVaadinApplication application,
			PortletRequest portletRequest) {
		ItcbTrackBean itcbTrackBean = new ItcbTrackBean();
		itcbTrackBean.setClassName(NET_SF_ITCB_ADDONS_MONITORING_TRACK_PORTLET_INTERCEPTOR_TRACKER_REQUEST_INTERCEPTOR);
	    itcbTrackBean.setMethod(HANDLE_REQUEST);
	    itcbTrackBean.setLayerType(LayerType.VAADIN_REQUEST);
		itcbTrackBean.setTrackIfNoChild(false);
		monitoringTransactionManager.setCurrentTrackBean(itcbTrackBean);
	}

	@Override
	public void handleRequestEnd(ItcbVaadinApplication application,
			HttpServletRequest servletRequest) {
		monitoringTransactionManager.endTransaction();
	}

	@Override
	public void handleRequestEnd(ItcbVaadinApplication application,
			PortletRequest portletRequest) {
	    monitoringTransactionManager.endTransaction();
	}

	@Override
	public void handleExceptionOnRequest(ItcbVaadinApplication application, Throwable e) {
		monitoringTransactionManager.getCurrentTrackBean().setException(e);
	}

}
